"""
Radiant

"""
# Prevent this file to be imported from Brython
import sys
try:
    import browser
    sys.exit()
except:
    pass

import os
import json
import jinja2
import pathlib
import importlib.util
from xml.etree import ElementTree
from typing import Union, List, Tuple, Optional

from tornado.web import Application, url, RequestHandler, StaticFileHandler
from tornado.ioloop import IOLoop
from tornado.httpserver import HTTPServer

RadiantAPI = object
DEBUG = True
PATH = Union[str, pathlib.Path]
URL = str


########################################################################
class PythonHandler(RequestHandler):
    def post(self):
        name = self.get_argument('name')
        args = tuple(json.loads(self.get_argument('args')))
        kwargs = json.loads(self.get_argument('kwargs'))

        if v := getattr(self, name, None)(*args, **kwargs):
            if v is None:
                data = json.dumps({'__RDNT__': 0, })
            else:
                data = json.dumps({'__RDNT__': v, })
        self.write(data)

    # ----------------------------------------------------------------------
    def test(self):
        """"""
        return True


########################################################################
class ThemeHandler(RequestHandler):

    # ----------------------------------------------------------------------
    def get(self):
        theme = self.get_theme()
        loader = jinja2.FileSystemLoader(os.path.join(
            os.path.dirname(__file__), 'templates'))
        env = jinja2.Environment(autoescape=True, loader=loader)
        env.filters['vector'] = self.hex2vector
        stylesheet = env.get_template('theme.css.template')
        self.write(stylesheet.render(**theme))

    # ----------------------------------------------------------------------
    @staticmethod
    def hex2vector(hex_: str):
        return ', '.join([str(int(hex_[i:i + 2], 16)) for i in range(1, 6, 2)])

    # ----------------------------------------------------------------------
    def get_theme(self):
        theme = self.settings['theme']

        if (not theme) or (not os.path.exists(theme)):
            theme = os.path.join(os.path.dirname(__file__),
                                 'templates', 'default_theme.xml')

        tree = ElementTree.parse(theme)
        theme_css = {child.attrib['name']                     : child.text for child in tree.getroot()}
        return theme_css


########################################################################
class RadiantHandler(RequestHandler):

    def initialize(self, **kwargs):
        self.initial_arguments = kwargs

    def get(self):
        variables = self.settings.copy()
        variables.update(self.initial_arguments)

        variables['argv'] = json.dumps(variables['argv'])
        self.render(f"{os.path.realpath(variables['template'])}", **variables)


# ----------------------------------------------------------------------
def make_app(class_: str, /,
             template: PATH = os.path.join(os.path.dirname(
                 __file__), 'templates', 'index.html'),
             environ: dict = {},
             mock_imports: Tuple[str] = [],
             handlers: Tuple[URL, Union[List[Union[PATH, str]],
                                        RequestHandler], dict] = (),
             python: Tuple[PATH, str] = (),
             theme: PATH = None,
             path: PATH = None,
             autoreload: bool = False):
    """
    Parameters
    ----------
    class_ :
        The main class name as string.
    template :
        Path for HTML file with the template.
    environ :
        Dictionary with arguments accessible from the template and main class.
    mock_imports :
        List with modules that exist in Python but not in Brython, this prevents
        imports exceptions.
    handlers :
        Custom handlers for server.
    python :
        Real Python scripting handler.
    theme :
        Path for the XML file with theme colors.
    path :
        Custom directory accesible from Brython PATH.
    autoreload :
        Activate the `autoreload` Tornado feature.
    """

    settings = {
        "debug": DEBUG,
        'static_path': os.path.join(os.path.dirname(__file__), 'brython-3.9.1'),
        'static_url_prefix': '/static/',
        "xsrf_cookies": False,
        'autoreload': autoreload,
    }

    environ.update({
        'class_': class_,
        'python_': python if python else (None, None),
        'module': os.path.split(sys.path[0])[-1],
        'file': os.path.split(sys.argv[0])[-1].removesuffix('.py'),
        'theme': theme,
        'argv': sys.argv,
        'template': template,
        'mock_imports': mock_imports,
        'path': path,
    })

    app = [
        url(r'^/$', RadiantHandler, environ),
        url(r'^/theme.css$', ThemeHandler),
        url(r'^/root/(.*)', StaticFileHandler, {'path': sys.path[0]}),
    ]

    if python:
        spec = importlib.util.spec_from_file_location(
            '.'.join(python).replace('.py', ''), os.path.abspath(python[0]))
        foo = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(foo)
        app.append(url(r'^/python_handler', getattr(foo, python[1])))

    for handler in handlers:
        if isinstance(handler[1], tuple):
            spec = importlib.util.spec_from_file_location(
                '.'.join(handler[1]).replace('.py', ''), os.path.abspath(handler[1][0]))
            foo = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(foo)
            app.append(url(handler[0], getattr(foo, handler[1][1]), handler[2]))
        else:
            app.append(url(*handler))

    if path:
        app.append(url(r'^/path/(.*)', StaticFileHandler, {'path': path}),)

    settings.update(environ)

    return Application(app, **settings)


# ----------------------------------------------------------------------
def RadiantServer(class_: str, /,
                  host: str = 'localhost',
                  port: str = '5000',
                  template: PATH = os.path.join(os.path.dirname(
                      __file__), 'templates', 'index.html'),
                  environ: dict = {},
                  mock_imports: Tuple[str] = [],
                  handlers: Tuple[URL, Union[List[Union[PATH, str]],
                                             RequestHandler], dict] = (),
                  python: Tuple[PATH, str] = (),
                  theme: Optional[PATH] = None,
                  path: Optional[PATH] = None,
                  autoreload: Optional[bool] = False,
                  callbacks: Optional[tuple] = (),
                  ):
    """Python implementation for move `class_` into a Bython environment.

    Configure the Tornado server and the Brython environment for run the
    `class_` in both frameworks at the same time.

    Parameters
    ----------
    class_ :
        The main class name as string.
    host :
        The host for server.
    port :
        The port for server.
    template :
        Path for HTML file with the template.
    environ :
        Dictionary with arguments accessible from the template and main class.
    mock_imports :
        List with modules that exist in Python but not in Brython, this prevents
        imports exceptions.
    handlers :
        Custom handlers for server.
    python :
        Real Python scripting handler.
    theme :
        Path for the XML file with theme colors.
    path :
        Custom directory accesible from Brython PATH.
    autoreload :
        Activate the `autoreload` Tornado feature.

    """

    print("Radiant server running on port {}".format(port))
    application = make_app(class_, python=python, template=template,
                           handlers=handlers, theme=theme, environ=environ,
                           mock_imports=mock_imports, path=path)
    http_server = HTTPServer(application)
    http_server.listen(port, host)

    for handler in callbacks:
        if isinstance(handler, tuple):
            spec = importlib.util.spec_from_file_location(
                '.'.join(handler).replace('.py', ''), os.path.abspath(handler[0]))
            foo = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(foo)
            getattr(foo, handler[1])()
            # app.append(url(handler[0], getattr(foo, handler[1][1]), handler[2]))
        else:
            handler()

    IOLoop.instance().start()





"""
Brython MDCComponent: MDCIcon
===============================
"""
from .core import MDCTemplate


########################################################################
class MDCIcon(MDCTemplate):
    """"""

    NAME = None, None

    MDC_optionals = {

        # Icons
        'icon': '<i class="material-icons mdc-button__icon" {reversed} aria-hidden="true">{icon}</i>',
        'fa_icon': '<i class="mdc-button__icon {fa_style} {fa_icon}"></i>',

    }

    # ----------------------------------------------------------------------
    def __new__(self, icon, size=None, **kwargs):
        """"""
        kwargs.update(self.format_icon(icon))
        del icon

        self.element = self.render(locals(), kwargs)
        return self.element

    # ----------------------------------------------------------------------
    @classmethod
    def __html__(cls, **context):
        """"""
        code = """
            {icon}
            {fa_icon}
            """

        return cls.render_html(code, context)




import sys
from . import RadiantAPI, RadiantServer, RadiantHandler


class fake:
    def __getattr__(self, attr):
        return None


brython = ['browser']
for module in brython:
    sys.modules[f"{module}"] = fake()

modules = ['sound']
for module in modules:
    sys.modules[f"radiant.{module}"] = fake()

components = ['MDCButton', 'MDCChips', 'MDCDialog', 'MDCFormField', 'MDCIcon',
              'MDCLayoutGrid', 'MDCList', 'MDCShape', 'MDCTab', 'MDCCard',
              'MDCComponent', 'MDCDrawer', 'MDCGridList', 'MDCImageList',
              'MDCLinearProgress', 'MDCMenu', 'MDCSnackbar', 'MDCTopAppBar']
for component in components:
    sys.modules[f"mdc.{component}"] = fake()





"""
Brython MDCComponent: MDCFormField
==================================


"""

from .core import MDCTemplate
from browser import html

########################################################################


class MDCFormField(MDCTemplate):
    """"""
    NAME = 'formField', 'MDCFormField'

    MDC_optionals = {
        'end': 'mdc-form-field--align-end',
    }

    # ----------------------------------------------------------------------

    def __new__(self, **kwargs):
        """"""
        self.element = self.render(locals(), kwargs)

        self.element.style = {'width': '100%',
                              'min-height': '48px',
                              # 'margin-bottom': '8px',
                              }

        return self.element

    # ----------------------------------------------------------------------

    @classmethod
    def __html__(cls, **context):
        """"""
        code = """
            <div class="mdc-form-field {end}"></div>
        """
        return cls.render_html(code, context)


########################################################################
class MDCForm(MDCTemplate):
    """"""

    # ----------------------------------------------------------------------
    def __new__(self, separator=None, formfield=None, formfield_style={}, **kwargs):
        """"""
        self.separator = separator
        self.formfield = formfield
        self.formfield_style = formfield_style
        # if formfield_style:
            # self.formfield = MDCFormField()
            # self.formfield.style = formfield_style

        self.element = self.render(locals(), kwargs)
        return self.element

    # ----------------------------------------------------------------------
    @classmethod
    def __html__(cls, **context):
        """"""
        code = """
            <form></form>
        """
        return cls.render_html(code, context)

    # ----------------------------------------------------------------------
    @classmethod
    def __genericElement__(cls, parent, Element, extern_label, *args, **kwargs):
        """"""
        if cls.formfield:
            formfield = cls.formfield.clone()
        else:
            formfield = MDCFormField()
            formfield.style = cls.formfield_style

        # elif cls.formfield_style:
            # formfield = MDCFormField(style=cls.formfield_style)

        # if 'formfield' in kwargs:
            # formfield = kwargs['formfield']

        el = Element(*args, **kwargs)
        formfield <= el
        if extern_label:
            formfield <= __formLabel__(id_=el.mdc.get_id(), label=args[0])
        if kwargs.get('helper_text', False):
            # print(kwargs.get('helper_text_persistent'))
            formfield <= __formHelper__(id_=el.mdc.get_id(), label=kwargs.get(
                'helper_text'), persistent=kwargs.get('helper_text_persistent', True))
        # cls.element <= formfield
        parent <= formfield

        if cls.separator:
            # cls.element <= cls.separator.clone()
            parent <= cls.separator.clone()

        return el, formfield

    # ----------------------------------------------------------------------

    @classmethod
    def Checkbox(cls, element, *args, **kwargs):
        """"""
        el, element = cls.__genericElement__(
            element, MDCCheckbox, True, *args, **kwargs)
        return el

    # ----------------------------------------------------------------------

    @classmethod
    def Radio(cls, element, *args, **kwargs):
        """"""
        el, element = cls.__genericElement__(
            element, MDCRadio, True, *args, **kwargs)

        # element.input = el
        return el

    # ----------------------------------------------------------------------

    @classmethod
    def Select(cls, element, *args, **kwargs):
        """"""
        el, element = cls.__genericElement__(
            element, MDCSelect, False, *args, **kwargs)
        return el

    # ----------------------------------------------------------------------

    @classmethod
    def Slider(cls, element, *args, **kwargs):
        """"""
        el, element = cls.__genericElement__(
            element, MDCSlider, False, *args, **kwargs)
        return el
    # ----------------------------------------------------------------------

    @classmethod
    def RangeSlider(cls, element, *args, **kwargs):
        """"""
        el, element = cls.__genericElement__(
            element, MDCRangeSlider, False, *args, **kwargs)
        return el

    # ----------------------------------------------------------------------

    @classmethod
    def Switch(cls, element, *args, **kwargs):
        """"""
        el, element = cls.__genericElement__(
            element, MDCSwitch, True, *args, **kwargs)
        return el

    # ----------------------------------------------------------------------

    @classmethod
    def TextField(cls, element, *args, **kwargs):
        """"""
        el, element = cls.__genericElement__(
            element, MDCTextField, False, *args, **kwargs)
        element.style = {'display': 'flow-root'}
        el.style = {'margin-top': 'unset'}
        return el

    # ----------------------------------------------------------------------

    @classmethod
    def TextAreaField(cls, element, *args, **kwargs):
        """"""
        el, element = cls.__genericElement__(
            element, MDCTextAreaField, False, *args, **kwargs)
        element.style = {'display': 'flow-root'}
        el.style = {'margin-top': '8px'}
        return el


########################################################################
class __formLabel__(MDCTemplate):
    """"""

    # ----------------------------------------------------------------------
    def __new__(self, id_, label):
        """"""
        code = """
            <label for="{id_}">{label}</label>
            """.format(id_=id_, label=label)
        return self.render_str(code)


########################################################################
class __formHelper__(MDCTemplate):
    """"""
    NAME = 'textField', 'MDCTextFieldHelperText'

    # ----------------------------------------------------------------------
    def __new__(self, id_, label, persistent=True):
        """"""
        if persistent:
            persistent = 'mdc-text-field-helper-text--persistent'
        else:
            persistent = ''
        code = """
            <p id="{id_}" class="mdc-text-field-helper-text {persistent}" aria-hidden="true">
              {label}
            </p>
            """.format(id_=id_, label=label, persistent=persistent)
        return self.render_str(code)


########################################################################
class MDCCheckbox(MDCTemplate):
    """"""
    NAME = 'checkbox', 'MDCCheckbox'

    MDC_optionals = {

        'disabled': 'mdc-checkbox--disabled',
        'checked': 'checked',

    }

    # ----------------------------------------------------------------------

    def __new__(self, label, name='', value='', checked=False, disabled=False, **kwargs):
        """"""
        self.element = self.render(locals(), kwargs)
        # self.element.need_label = True
        return self.element

    # ----------------------------------------------------------------------

    @classmethod
    def __html__(cls, **context):
        """"""

        cls.ID = cls.new_id()
        context['id'] = cls.ID

        code = """
            <div class="mdc-checkbox  {disabled}">
              <input type="checkbox" class="mdc-checkbox__native-control"
              id="{id}
              name="{name}"
              value="{value}"
              {checked}
              "/>
              <div class="mdc-checkbox__background">
                <svg class="mdc-checkbox__checkmark"
                     viewBox="0 0 24 24">
                  <path class="mdc-checkbox__checkmark-path"
                        fill="none"
                        d="M1.73,12.91 8.1,19.28 22.79,4.59"/>
                </svg>
                <div class="mdc-checkbox__mixedmark"></div>
              </div>
              <div class="mdc-checkbox__ripple"></div>
            </div>
        """
        return cls.render_html(code, context)


########################################################################
class MDCRadio(MDCTemplate):
    """"""
    NAME = 'radio', 'MDCRadio'

    MDC_optionals = {

        'disabled': 'mdc-radio--disabled',
        'checked': 'checked',

    }

    # ----------------------------------------------------------------------

    def __new__(self, label, name='', checked=False, disabled=False, **kwargs):
        """"""
        self.element = self.render(locals(), kwargs)
        # self.element.need_label = True
        return self.element

    # ----------------------------------------------------------------------
    @classmethod
    def __html__(cls, **context):
        """"""

        cls.ID = cls.new_id()
        context['id'] = cls.ID

        if context['disabled']:
            context['input_disabled'] = 'disabled'
        else:
            context['input_disabled'] = ''

        code = """
            <div class="mdc-radio {disabled}">
              <input class="mdc-radio__native-control" type="radio" id="{id}" name="{name}" {input_disabled} {checked}>
              <div class="mdc-radio__background">
                <div class="mdc-radio__outer-circle"></div>
                <div class="mdc-radio__inner-circle"></div>
              </div>
              <div class="mdc-radio__ripple"></div>
            </div>
        """
        return cls.render_html(code, context)


########################################################################
class MDCSelect(MDCTemplate):
    """"""
    NAME = 'select', 'MDCSelect'

    MDC_optionals = {

        'disabled': 'mdc-select--disabled',
        'filled': 'mdc-select--filled',
        'outline': 'mdc-select--outline',

    }

    # ----------------------------------------------------------------------

    def __new__(self, label, options=[], selected=None, disabled=False, filled=False, outline=False, **kwargs):
        """"""
        self.element = self.render(locals(), kwargs, attach_now=False)
        # self.element.need_label = False

        if (not filled) and (not outline):
            filled = True

        if options:
            self.add_options(self.element, options, selected)
        self.attach()

        return self.element

    # ----------------------------------------------------------------------

    @classmethod
    def __html__(cls, **context):
        """"""

        cls.ID = cls.new_id()
        context['id'] = cls.ID

        # print('3' * 70)
        # print(context['outline'])
        # print(context['filled'])

        if context['outline']:
            code = """

                <div class="mdc-select mdc-select--outlined">
                  <div class="mdc-select__anchor" aria-labelledby="outlined-select-label">
                    <span class="mdc-notched-outline">
                      <span class="mdc-notched-outline__leading"></span>
                      <span class="mdc-notched-outline__notch">
                        <span id="outlined-select-label" class="mdc-floating-label">{label}</span>
                      </span>
                      <span class="mdc-notched-outline__trailing"></span>
                    </span>
                    <span class="mdc-select__selected-text-container">
                      <span id="demo-selected-text" class="mdc-select__selected-text">{selected}</span>
                    </span>


                    <span class="mdc-select__dropdown-icon">
                      <svg
                          class="mdc-select__dropdown-icon-graphic"
                          viewBox="7 10 10 5" focusable="false">
                        <polygon
                            class="mdc-select__dropdown-icon-inactive"
                            stroke="none"
                            fill-rule="evenodd"
                            points="7 10 12 15 17 10">
                        </polygon>
                        <polygon
                            class="mdc-select__dropdown-icon-active"
                            stroke="none"
                            fill-rule="evenodd"
                            points="7 15 12 10 17 15">
                        </polygon>
                      </svg>
                    </span>
                  </div>

                    <div class="mdc-select__menu mdc-menu mdc-menu-surface mdc-menu-surface--fullwidth">
                      <ul class="mdc-list" role="listbox">
                      </ul>
                    </div>
                </div>
            """

        else:
            code = """
                <div class="mdc-select mdc-select--filled">
                  <div class="mdc-select__anchor"
                       role="button"
                       aria-haspopup="listbox"
                       aria-expanded="false"
                       aria-labelledby="demo-label demo-selected-text">

                    <span class="mdc-select__ripple"></span>
                    <span id="demo-label" class="mdc-floating-label">{label}</span>

                    <span class="mdc-select__selected-text-container">
                      <span id="demo-selected-text" class="mdc-select__selected-text">{selected}</span>
                    </span>
                    <span class="mdc-select__dropdown-icon">
                      <svg
                          class="mdc-select__dropdown-icon-graphic"
                          viewBox="7 10 10 5" focusable="false">
                        <polygon
                            class="mdc-select__dropdown-icon-inactive"
                            stroke="none"
                            fill-rule="evenodd"
                            points="7 10 12 15 17 10">
                        </polygon>
                        <polygon
                            class="mdc-select__dropdown-icon-active"
                            stroke="none"
                            fill-rule="evenodd"
                            points="7 15 12 10 17 15">
                        </polygon>
                      </svg>
                    </span>
                    <span class="mdc-line-ripple"></span>
                  </div>

                  <div class="mdc-select__menu mdc-menu mdc-menu-surface mdc-menu-surface--fullwidth">
                    <ul class="mdc-list" role="listbox">
                    </ul>

                  </div>
                </div>
            """

        return cls.render_html(code, context)

    # ----------------------------------------------------------------------
    @classmethod
    def get(self, name):
        """"""
        if name is 'options_placeholder':
            return self.element.select('ul.mdc-list')[0]
        elif name is 'label':
            return self.element.select('.mdc-select__selected-text')[0]
        elif name is 'value':
            return self.element.select('.mdc-select__selected-text')[0].text

    # ----------------------------------------------------------------------
    @classmethod
    def add_option(cls, element, label, value, selected=False, disabled=False):
        """"""
        option = __selectItem__(label, value=value,
                                selected=selected, disabled=disabled)

        cls.get('options_placeholder') <= option
        return option

    # ----------------------------------------------------------------------
    @classmethod
    def add_options(cls, element, options, selected=None):
        """"""
        # if selected is None:
            # cls.add_option(element, '', '', selected=True)
        # else:
            # cls.get('label').class_name += cls.get('label').class_name.replace('mdc-list-item--selected', '')

        # if selected is None:
            # selected = options[0][1]

        if selected:
            cls.get('label').text = selected

        for option in options:
            cls.add_option(element, *option, option[1] == selected)


########################################################################
class __selectItem__(MDCTemplate):
    """"""

    MDC_optionals = {

        'selected': 'mdc-list-item--selected',

        'disable': 'mdc-list-item--disabled',
        # 'stack' = '{}'
        # 'icon': '<i class="radiant-menu-icon material-icons" aria-hidden="true" style="margin-right: 15px;">{icon}</i>',
        # 'fa_icon': '<i class="radiant-menu-icon {fa_style} {fa_icon}" style="margin-right: 15px;"></i>',

    }

    # ----------------------------------------------------------------------
    def __new__(self, label, value, selected=False, disable=False, **kwargs):
        """"""

        self.element = self.render(locals(), kwargs)

        return self.element

    # ----------------------------------------------------------------------
    @classmethod
    def __html__(cls, **context):
        """"""
        if context['selected']:
            context['aria_selected'] = 'aria-selected="true"'
        else:
            context['aria_selected'] = 'aria-selected="false"'

        if context['disable']:
            context['aria_disabled'] = 'aria-selected="true"'
        else:
            context['aria_disabled'] = 'aria-selected="false"'

        code = """
            <li class="mdc-list-item {disable} {selected}" {aria_selected} data-value="{value}" {aria_disabled} role="option">
              <span class="mdc-list-item__ripple"></span>
              <span class="mdc-list-item__text">
                {label}
              </span>
            </li>
        """
        return cls.render_html(code, context)

    # # ----------------------------------------------------------------------
    # @classmethod
    # def get(self, name):
        # """"""
        # if name is 'icon':
            # return self.element.select('.radiant-menu-icon')[0]


########################################################################
class MDCSlider(MDCTemplate):
    """"""
    NAME = 'slider', 'MDCSlider'

    MDC_optionals = {

        'disabled': 'mdc-slider--disabled',
        'marks': 'mdc-slider--tick-marks',

    }

    # ----------------------------------------------------------------------
    def __new__(self, label, min=0, max=100, value=50, step=1, discrete=False, marks=False, continuous=False, disabled=False, **kwargs):
        """"""

        if not discrete and not continuous:
            continuous = True

        self.element = self.render(locals(), kwargs)
        # self.element.need_label = False

        return self.element

    # ----------------------------------------------------------------------
    @classmethod
    def __html__(cls, **context):
        """"""

        cls.ID = cls.new_id()
        context['id'] = cls.ID

        if context['disabled']:
            context['input_disabled'] = 'disabled'
        else:
            context['input_disabled'] = ''

        if context.get('continuous'):
            code = """

                <div class="mdc-slider {disabled}">
                  <input class="mdc-slider__input" {input_disabled} type="range" min="{min}" max="{max}" value="{value}" step="{step}" name="name-{id}" aria-label="Continuous slider demo">
                  <div class="mdc-slider__track">
                    <div class="mdc-slider__track--inactive"></div>
                    <div class="mdc-slider__track--active">
                      <div class="mdc-slider__track--active_fill"></div>
                    </div>
                  </div>
                  <div class="mdc-slider__thumb">
                    <div class="mdc-slider__thumb-knob"></div>
                  </div>
                </div>

            """

        elif context.get('discrete'):
            code = """
                <div class="mdc-slider {disabled} mdc-slider--discrete {marks}">
                  <input class="mdc-slider__input" {input_disabled} type="range" min="{min}" max="{max}" value="{value}" "name-{id}" step="{step}" aria-label="Discrete slider demo">
                  <div class="mdc-slider__track">
                    <div class="mdc-slider__track--inactive"></div>
                    <div class="mdc-slider__track--active">
                      <div class="mdc-slider__track--active_fill"></div>
                    </div>
                  </div>
                  <div class="mdc-slider__thumb">
                    <div class="mdc-slider__value-indicator-container">
                      <div class="mdc-slider__value-indicator">
                        <span class="mdc-slider__value-indicator-text">
                          {value}
                        </span>
                      </div>
                    </div>
                    <div class="mdc-slider__thumb-knob"></div>
                  </div>
                </div>
            """

        return cls.render_html(code, context)


########################################################################
class MDCRangeSlider(MDCTemplate):
    """"""
    NAME = 'slider', 'MDCSlider'

    MDC_optionals = {

        'disabled': 'mdc-slider--disabled',

    }

    # ----------------------------------------------------------------------
    def __new__(self, label, min=0, max=100, value_lower=0, value_upper=100, step=1, disabled=False, **kwargs):
        """"""
        self.element = self.render(locals(), kwargs)
        # self.element.need_label = False

        return self.element

    # ----------------------------------------------------------------------
    @classmethod
    def __html__(cls, **context):
        """"""

        cls.ID = cls.new_id()
        context['id'] = cls.ID

        if context['disabled']:
            context['input_disabled'] = 'disabled'
        else:
            context['input_disabled'] = ''

        code = """
            <div class="mdc-slider mdc-slider--range {disabled}">
              <input class="mdc-slider__input" {input_disabled} type="range" min="{min}", step="{step}" max="{max}" value="{value_lower}" name="rangeStart-{id}" aria-label="Range lower slider demo">
              <input class="mdc-slider__input" {input_disabled} type="range" min="{min}", step="{step}" max="{max}" value="{value_upper}" name="rangeEnd-{id}" aria-label="Range upper slider demo">
              <div class="mdc-slider__track">
                <div class="mdc-slider__track--inactive"></div>
                <div class="mdc-slider__track--active">
                  <div class="mdc-slider__track--active_fill"
                       style="transform:scaleX(.4); left:30%"></div>
                </div>
              </div>
              <div class="mdc-slider__thumb" style="left:calc(30%-24px)">
                <div class="mdc-slider__thumb-knob"></div>
              </div>
              <div class="mdc-slider__thumb" style="left:calc(70%-24px)">
                <div class="mdc-slider__thumb-knob"></div>
              </div>
            </div>
        """

        return cls.render_html(code, context)


########################################################################
class MDCSwitch(MDCTemplate):
    """"""
    NAME = 'switchControl', 'MDCSwitch'

    MDC_optionals = {

        'disabled': 'disabled',
        # 'box': 'mdc-select--box',
        'checked': 'mdc-switch--checked',

    }

    # ----------------------------------------------------------------------

    def __new__(self, label, checked=True, **kwargs):
        """"""
        self.element = self.render(locals(), kwargs)
        return self.element

    # ----------------------------------------------------------------------

    @classmethod
    def __html__(cls, **context):
        """"""

        cls.ID = cls.new_id()
        context['id'] = cls.ID

        if context.get('checked'):
            context['native_checked'] = 'checked'
        else:
            context['native_checked'] = ''

        code = """
        <div class="mdc-switch {checked}">
          <div class="mdc-switch__track"></div>
          <div class="mdc-switch__thumb-underlay">
            <div class="mdc-switch__thumb">
                <input type="checkbox" id="{id}" class="mdc-switch__native-control" role="switch" {native_checked}>
            </div>
          </div>
        </div>
        """

        return cls.render_html(code, context)


########################################################################
class MDCTextField(MDCTemplate):
    """"""
    NAME = 'textField', 'MDCTextField'

    MDC_optionals = {

        'disabled': 'mdc-text-field--disabled',
        # 'box': 'mdc-select--box',
        # 'fullwidth': 'mdc-text-field--fullwidth',

        'value': 'value="{value}"',


    }

    # ----------------------------------------------------------------------

    def __new__(self, label, value=False, type='text', leading_icon=False, trailing_icon=False, helper_text=False, helper_text_persistent=True, outlined=False, disabled=False, fullwidth=False, **kwargs):
        """"""

        self.element = self.render(locals(), kwargs)
        return self.element

    # ----------------------------------------------------------------------

    @classmethod
    def __html__(cls, **context):
        """"""
        cls.ID = cls.new_id()
        context['id'] = cls.ID

        if context['disabled']:
            context['input_disabled'] = 'disabled'
        else:
            context['input_disabled'] = ''

        if context['value']:
            context['upgraded'] = 'mdc-text-field--upgraded'
            context['float_above'] = 'mdc-floating-label--float-above'
        else:
            context['upgraded'] = ''
            context['float_above'] = ''

        if context['helper_text']:
            context['helper_text'] = 'aria-controls="username-helper-text" aria-describedby="username-helper-text"'
        else:
            context['helper_text'] = ''

        if context['leading_icon']:
            context['icon'] = '<i class="material-icons mdc-text-field__icon" tabindex="0" role="button">{icon}</i>'.format(
                icon=context['leading_icon'])
            context['leading_icon'] = 'mdc-text-field--with-leading-icon'

        elif context['trailing_icon']:
            context['icon'] = '<i class="material-icons mdc-text-field__icon" tabindex="0" role="button">{icon}</i>'.format(
                icon=context['trailing_icon'])
            context['trailing_icon'] = 'mdc-text-field--with-trailing-icon'

        else:
            context['leading_icon'] = ''
            context['trailing_icon'] = ''
            context['icon'] = ''

        if context.get('outlined'):
            code = """
                <div class="mdc-text-field mdc-text-field--outlined {disabled} {upgraded} {leading_icon} {trailing_icon}">
                    {icon}
                  <input type="{type}" id="{id}" class="mdc-text-field__input" {value}>
                  <div class="mdc-notched-outline">
                    <div class="mdc-notched-outline__leading"></div>
                    <div class="mdc-notched-outline__notch">
                      <label for="{id}" class="mdc-floating-label {float_above}">{label}</label>
                    </div>
                    <div class="mdc-notched-outline__trailing"></div>
                  </div>
                </div>
            """

        elif context.get('fullwidth'):
            code = """
            <div class="mdc-text-field mdc-text-field--fullwidth {upgraded} {disabled}">
              <input class="mdc-text-field__input" id="{id}" type="{type}" {value} placeholder="{label}" aria-label="{label}" {helper_text}>
            </div>
            """

        else:
            code = """
                <div class="mdc-text-field {disabled} {upgraded} {leading_icon} {trailing_icon}">
                    {icon}
                  <input type="{type}" id="{id}" class="mdc-text-field__input" {input_disabled} {value} {helper_text}>
                  <label class="mdc-floating-label" for="{id}">{label}</label>
                  <div class="mdc-line-ripple"></div>
                </div>
            """

        return cls.render_html(code, context)


########################################################################
class MDCTextAreaField(MDCTemplate):
    """"""
    NAME = 'textField', 'MDCTextField'

    MDC_optionals = {

        'disabled': 'mdc-text-field--disabled',
        # 'dense': 'mdc-text-field--dense',
        # 'box': 'mdc-select--box',
        'fullwidth': 'mdc-text-field--fullwidth',

        # 'value': 'value="{value}"',


    }

    # ----------------------------------------------------------------------

    def __new__(self, label, value='', rows='2', cols='40', helper_text=False, helper_text_persistent=True, disabled=False, fullwidth=False, **kwargs):
        """"""

        self.element = self.render(locals(), kwargs)
        return self.element

    # ----------------------------------------------------------------------

    @classmethod
    def __html__(cls, **context):
        """"""
        cls.ID = cls.new_id()
        context['id'] = cls.ID

        if context['disabled']:
            context['input_disabled'] = 'disabled'
        else:
            context['input_disabled'] = ''

        if context['helper_text']:
            context['helper_text'] = 'aria-controls="username-helper-text" aria-describedby="username-helper-text"'
        else:
            context['helper_text'] = ''

        code = """
        <div class="mdc-text-field mdc-text-field--textarea {fullwidth} {disabled}">
          <textarea id="{id}" class="mdc-text-field__input" rows="{rows}" cols="{cols}" {helper_text} {input_disabled}>{value}</textarea>

            <div class="mdc-notched-outline">
              <div class="mdc-notched-outline__leading"></div>
              <div class="mdc-notched-outline__notch">
                <label for="{id}" class="mdc-floating-label">{label}</label>
              </div>
              <div class="mdc-notched-outline__trailing"></div>
            </div>

        </div>
        """

        return cls.render_html(code, context)



"""
Brython MDCComponent: MDCButton
===============================
"""
from .core import MDCTemplate


########################################################################
class MDCButton(MDCTemplate):
    """"""

    NAME = 'button', 'MDCButton'

    CSS_classes = {

        'raised': 'mdc-button--raised',
        'unelevated': 'mdc-button--unelevated',
        'outlined': 'mdc-button--outlined',
        'dense': 'mdc-button--dense',

    }

    MDC_optionals = {

        'disabled': 'disabled',
        # 'reversed': 'style = "margin-left: 8px; margin-right: -4px;"',

        # Icons
        'icon': '<i class="material-icons mdc-button__icon" aria-hidden="true">{icon}</i>',
        'fa_icon': '<i class="mdc-button__icon {fa_style} {fa_icon}"></i>',


        'label': '<span class="mdc-button__label">{label}</span>',
        # <i class="material-icons mdc-button__icon" aria-hidden="true">favorite</i>


        # # Icon for icon only
        # 'icon_': '<button class="mdc-icon-button material-icons">{icon_}</button>',
        # 'fa_icon_': '<button class="mdc-icon-button {fa_style} {fa_icon}"></button>',

        # # Icons for a
        # 'icon_a': '<a class="mdc-icon-button material-icons">{icon_}</a>',
        # 'fa_icon_a': '<a class="mdc-icon-button {fa_style} {fa_icon}"></a>',


    }

    # ----------------------------------------------------------------------
    def __new__(self, label=None, href=None, icon=False, style_icon={}, **kwargs):
        """"""
        kwargs.update(self.format_icon(icon))
        del icon

        if kwargs.get('raised', False) or kwargs.get('outlined', False):
            kwargs['unelevated'] = False

        self.element = self.render(locals(), kwargs)

        try:
            self.element.select_one('i').style = style_icon
        except:
            pass

        return self.element

    # ----------------------------------------------------------------------
    @classmethod
    def __html__(cls, **context):
        """"""
        if context.get('href'):
            # if context.get('text'):
            if not context.get('reversed'):
                code = """
                    <a href="{href}" class="mdc-button {CSS_classes}" {disabled}>
                    <div class="mdc-button__ripple"></div>
                    {icon}
                    {fa_icon}
                    {label}
                    </a>
                """
            else:
                code = """
                    <a href="{href}" class="mdc-button {CSS_classes}" {disabled}>
                    <div class="mdc-button__ripple"></div>
                    {label}
                    {icon}
                    {fa_icon}
                    </a>
                """
            # else:
                # code = """
                # {icon_a}
                # {fa_icon_a}
                # """

        else:
            # if context.get('text'):
            if not context.get('reversed'):
                code = """
                    <button class="mdc-button {CSS_classes}" {disabled}>
                    <div class="mdc-button__ripple"></div>
                    {icon}
                    {fa_icon}
                    {label}
                    </button>
                """
            else:
                code = """
                    <button class="mdc-button {CSS_classes}" {disabled}>
                    <div class="mdc-button__ripple"></div>
                    {label}
                    {icon}
                    {fa_icon}
                    </button>

                """
            # else:
                # code = """
                    # {icon_}
                    # {fa_icon_}
                    # """

        return cls.render_html(code, context)


########################################################################
class MDCFab(MDCButton):
    """"""

    CSS_classes = {

        'mini': 'mdc-fab--mini',
        'exited': 'mdc-fab--exited',
        'extended': 'mdc-fab--extended',

    }

    MDC_optionals = {

        'label': '<span class="mdc-fab__label">{label}</span>',
        'disabled': 'disabled',


        'icon': '<i class="material-icons mdc-fab__icon" aria-hidden="true">{icon}</i>',
        'fa_icon': '<i class="mdc-fab__icon {fa_style} {fa_icon}"></i>',

        # 'icon': '<i class="material-icons mdc-button__icon" {reversed} aria-hidden="true">{icon}</i>',
        'fa_icon': '<i class="mdc-button__icon {fa_style} {fa_icon}"></i>',


    }

    # ----------------------------------------------------------------------
    def __new__(self, icon, label=False, **kwargs):
        """"""
        kwargs.update(self.format_icon(icon))
        del icon

        if label:
            extended = True
        else:
            extended = False

        self.element = self.render(locals(), kwargs)
        return self.element

    # ----------------------------------------------------------------------
    @classmethod
    def __html__(cls, **context):
        """"""
        if context.get('href'):

            if not context.get('reversed'):
                code = """
                    <a class="mdc-fab {CSS_classes}" href="{href}" {disabled}>
                      <div class="mdc-fab__ripple"></div>
                      {icon}
                      {fa_icon}
                      {label}
                    </a>
                """
            else:
                code = """
                    <a class="mdc-fab {CSS_classes}" href="{href}" {disabled}>
                      <div class="mdc-fab__ripple"></div>
                      {label}
                      {icon}
                      {fa_icon}
                    </a>
                """

        else:

            if not context.get('reversed'):
                code = """
                    <button class="mdc-fab {CSS_classes}" {disabled}>
                      <div class="mdc-fab__ripple"></div>
                      {icon}
                      {fa_icon}
                      {label}
                    </button>
                """
            else:
                code = """
                    <button class="mdc-fab {CSS_classes}" {disabled}>
                      <div class="mdc-fab__ripple"></div>
                      {label}
                      {icon}
                      {fa_icon}
                    </button>
                """

        return cls.render_html(code, context)


########################################################################
class MDCIconToggle(MDCTemplate):
    """"""

    NAME = 'iconButton', 'MDCIconButtonToggle'

    # ----------------------------------------------------------------------
    def __new__(self, icon_on, icon_off, state='off', **kwargs):
        """"""
        if state == 'on':
            icon_on, icon_off = icon_off, icon_on
        elif state == 'off':
            state = icon_off

        self.element = self.render(locals(), kwargs)
        return self.element

    # ----------------------------------------------------------------------
    @classmethod
    def __html__(cls, **context):
        """"""
        code = """

            <button class="mdc-icon-button" aria-pressed="false">
               <i class="material-icons mdc-icon-button__icon mdc-icon-button__icon--on">{icon_on}</i>
               <i class="material-icons mdc-icon-button__icon">{icon_off}</i>
            </button>

        """

        return cls.render_html(code, context)



"""
Brython MDCComponent: MDCCard
=============================


"""

from .core import MDCTemplate
from .MDCButton import MDCButton, MDCIconToggle

########################################################################


class MDCCard(MDCTemplate):
    """"""

    NAME = 'card', 'MDCCcard'

    MDC_optionals = {

        'outlined': 'mdc-card--outlined',
        'square': 'mdc-card__media--square',
        '_16_9': 'mdc-card__media--16-9',
        'full_bleed': 'mdc-card__actions--full-bleed',
        "primary_action": "mdc-card__primary-action",

    }

    # ----------------------------------------------------------------------
    def __new__(self, title='', subtitle='', media_content='', text_content='', square=True, **kwargs):
        """"""
        self.element = self.render(locals(), kwargs)
        return self.element

    # ----------------------------------------------------------------------
    @classmethod
    def __html__(cls, **context):
        """"""
        code = """
<div class="mdc-card">
  <div class="mdc-card__primary-action">
    <div class="mdc-card__media mdc-card__media--square">
      <div class="mdc-card__media-content">Title</div>
    </div>
    <!-- ... additional primary action content ... -->
  </div>
  <div class="mdc-card__actions">
    <div class="mdc-card__action-buttons">
      <button class="mdc-button mdc-card__action mdc-card__action--button">
        <div class="mdc-button__ripple"></div>
        <span class="mdc-button__label">Action 1</span>
      </button>
      <button class="mdc-button mdc-card__action mdc-card__action--button">
        <div class="mdc-button__ripple"></div>
        <span class="mdc-button__label">Action 2</span>
      </button>
    </div>
    <div class="mdc-card__action-icons">
      <button class="material-icons mdc-icon-button mdc-card__action mdc-card__action--icon" title="Share">share</button>
      <button class="material-icons mdc-icon-button mdc-card__action mdc-card__action--icon" title="More options">more_vert</button>
    </div>
  </div>
</div>
        """

        code = """
<div class="mdc-card demo-card">
  <div class="mdc-card__primary-action demo-card__primary-action" tabindex="0">
    <div class="mdc-card__media mdc-card__media--16-9 demo-card__media" style="background-image: url(&quot;https://material-components.github.io/material-components-web-catalog/static/media/photos/3x2/2.jpg&quot;);"></div>
    <div class="demo-card__primary">
      <h2 class="demo-card__title mdc-typography mdc-typography--headline6">Our Changing Planet</h2>
      <h3 class="demo-card__subtitle mdc-typography mdc-typography--subtitle2">by Kurt Wagner</h3>
    </div>
    <div class="demo-card__secondary mdc-typography mdc-typography--body2">Visit ten places on our planet that are undergoing the biggest changes today.</div>
  </div>
  <div class="mdc-card__actions">
    <div class="mdc-card__action-buttons">
      <button class="mdc-button mdc-card__action mdc-card__action--button">  <span class="mdc-button__ripple"></span> Read</button>
      <button class="mdc-button mdc-card__action mdc-card__action--button">  <span class="mdc-button__ripple"></span> Bookmark</button>
    </div>
    <div class="mdc-card__action-icons">
      <button class="mdc-icon-button mdc-card__action mdc-card__action--icon--unbounded" aria-pressed="false" aria-label="Add to favorites" title="Add to favorites">
        <i class="material-icons mdc-icon-button__icon mdc-icon-button__icon--on">favorite</i>
        <i class="material-icons mdc-icon-button__icon">favorite_border</i>
      </button>
      <button class="mdc-icon-button material-icons mdc-card__action mdc-card__action--icon--unbounded" title="Share" data-mdc-ripple-is-unbounded="true">share</button>
      <button class="mdc-icon-button material-icons mdc-card__action mdc-card__action--icon--unbounded" title="More options" data-mdc-ripple-is-unbounded="true">more_vert</button>
    </div>
  </div>
</div>
        """

        return cls.render_html(code, context)

    # ----------------------------------------------------------------------
    @classmethod
    def get(self, name):
        """"""
        if name is 'actions':
            return self.element.select('.mdc-card__actions')[0]

        elif name is 'title':
            return self.element.select('.radian-title')[0]

        elif name is 'subtitle':
            return self.element.select('.radian-subtitle')[0]

        elif name is 'header':
            return self.element.select('.radiant-title__header')[0]

        elif name is 'content':
            return self.element.select('.radian-content')[0]

        elif name is 'media':
            return self.element.select('.mdc-card__media')[0]

        elif name is 'media_content':
            return self.element.select('.mdc-card__media-content')[0]

        elif name is 'action_buttons':
            return self.element.select('.mdc-card__action-buttons')[0]

        elif name is 'action_icons':
            return self.element.select('.mdc-card__action-icons')[0]

    # ----------------------------------------------------------------------
    @classmethod
    def add_action_button(cls, element, *args, **kwargs):
        """"""
        button = MDCButton(*args, **kwargs)
        button.class_name += ' mdc-card__action mdc-card__action--button'
        cls.get('action_icons') <= button

        return button

    # ----------------------------------------------------------------------
    @classmethod
    def add_action_icontoggle(cls, element, *args, **kwargs):
        """"""
        button = MDCIconToggle(*args, **kwargs)
        button.class_name += ' mdc-card__action mdc-card__action--icon'
        cls.get('action_icons') <= button

        return button

    # ----------------------------------------------------------------------
    @classmethod
    def add_action_icon(cls, element, icon, *args, **kwargs):
        """"""
        button = MDCButton(icon=icon, *args, **kwargs)
        button.class_name += ' mdc-card__action mdc-card__action--icon'
        cls.get('action_icons') <= button

        return button



import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

setup(
    name='radiant',
    version='3.2',
    packages=['radiant'],

    author='Yeison Cardona',
    author_email='yencardonaal@unal.edu.co',
    maintainer='Yeison Cardona',
    maintainer_email='yencardonaal@unal.edu.co',

    download_url='https://github.com/UN-GCPDS/brython-radiant',

    install_requires=['Jinja2>=2.11.3'
                      'tornado>=6.1',
                      ],

    include_package_data=True,
    license='BSD-2-Clause',
    description="Brython Framework",

    long_description=README,
    long_description_content_type='text/markdown',

    python_requires='>=3.8',


    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'Topic :: Scientific/Engineering :: Human Machine Interfaces',
        'Topic :: Scientific/Engineering :: Medical Science Apps.',
        'Topic :: Software Development :: Embedded Systems',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Hardware :: Hardware Drivers',
    ],

)



